@@ -1,6 +1,7 @@
 import abc
 from rpy2.robjects.robject import RObjectMixin
 import rpy2.rinterface as rinterface
+from rpy2.rinterface_lib import sexp
 from . import conversion
 
 import rpy2.rlike.container as rlc
@@ -13,6 +14,7 @@ import jinja2
 import time
 import pytz
 import tzlocal
+from datetime import date
 from datetime import datetime
 from time import struct_time, mktime, tzname
 from operator import attrgetter
@@ -20,7 +22,7 @@ import warnings
 
 from rpy2.rinterface import (Sexp, ListSexpVector, StrSexpVector,
                              IntSexpVector, ByteSexpVector, BoolSexpVector,
-                             ComplexSexpVector,
+                             ComplexSexpVector, PairlistSexpVector,
                              FloatSexpVector, NA_Real, NA_Integer,
                              NA_Character, NA_Logical, NULL, MissingArg)
 
@@ -583,12 +585,17 @@ class FactorVector(IntVector):
             yield levels[x-1]
 
 
+class PairlistVector(Vector, PairlistSexpVector):
+    """R pairlist vector."""
+    pass
+
+
 class ListVector(Vector, ListSexpVector):
     """ R list (vector of arbitray elements)
 
-    ListVector(itemable) -> ListVector.
+    ListVector(iterable) -> ListVector.
 
-    The parameter 'itemable' can be:
+    The parameter 'iterable' can be:
 
     - an object with a method `items()`, such for example a dict,
       a rpy2.rlike.container.TaggedList,
@@ -717,11 +724,6 @@ class ListVector(Vector, ListSexpVector):
         return res
 
 
-class DateVector(FloatVector):
-    """ Vector of dates """
-    pass
-
-
 class POSIXt(abc.ABC):
     """ POSIX time vector. This is an abstract class. """
 
@@ -824,6 +826,36 @@ def get_timezone():
     return timezone
 
 
+class DateVector(FloatVector):
+    """ Representation of dates as number of days since 1/1/1970.
+
+    Date(seq) -> Date.
+
+    The constructor accepts either an R vector floats
+    or a sequence (an object with the Python
+    sequence interface) of time.struct_time objects.
+    """
+
+    def __init__(self, seq):
+        """ Create a POSIXct from either an R vector or a sequence
+        of Python datetime.date objects.
+        """
+
+        if isinstance(seq, Sexp):
+            init_param = seq
+        elif isinstance(seq[0], date):
+            init_param = DateVector.sexp_from_date(seq)
+        else:
+            raise TypeError(
+                'Unable to create an R Date vector from objects of type %s' %
+                type(seq))
+        super().__init__(init_param)
+
+    @classmethod
+    def sexp_from_date(cls, seq):
+        return cls(FloatVector([x.toordinal() for x in seq]))
+
+
 class POSIXct(POSIXt, FloatVector):
     """ Representation of dates as seconds since Epoch.
     This form is preferred to POSIXlt for inclusion in a DataFrame.
@@ -914,23 +946,31 @@ class POSIXct(POSIXt, FloatVector):
 
         return POSIXct._sexp_from_seq(seq, attrgetter('tzinfo'), f)
 
+    @staticmethod
+    def isrinstance(obj) -> bool:
+        """Is an R object an instance of POSIXct."""
+        return obj.rclass[0] == 'POSIXct'
+
     def iter_localized_datetime(self):
         """Iterator yielding localized Python datetime objects."""
         try:
-            tzone_name = self.do_slot('tzone')[0]
+            r_tzone_name = self.do_slot('tzone')[0]
         except LookupError:
             warnings.warn('R object inheriting from "POSIXct" but without '
                           'attribute "tzone".')
-            tzone_name = ''
-        if tzone_name == '':
+            r_tzone_name = ''
+        if r_tzone_name == '':
             # R is implicitly using the local timezone, while Python
             # time libraries will assume UTC.
-            tzone = get_timezone()
+            r_tzone = get_timezone()
         else:
-            tzone = pytz.timezone(tzone_name)
+            r_tzone = pytz.timezone(r_tzone_name)
+
         for x in self:
-            yield (None if math.isnan(x)
-                   else tzone.localize(datetime.fromtimestamp(x)))
+            yield (
+                None if math.isnan(x)
+                else datetime.fromtimestamp(x, r_tzone)
+            )
 
 
 class Array(Vector):
@@ -953,10 +993,10 @@ class Array(Vector):
         value = conversion.py2rpy(value)
         self._dim_set(self, value)
 
-    dim = property(__dim_get, __dim_set,
+    dim = property(__dim_get, __dim_set, None,
                    "Get or set the dimension of the array.")
 
-    def __dimnames_get(self):
+    def __dimnames_get(self) -> sexp.Sexp:
         """ Return a list of name vectors
         (like the R function 'dimnames' does)."""
 
@@ -1157,15 +1197,16 @@ class DataFrame(ListVector):
         </table>
     """)
 
-    def __init__(self, obj, stringsasfactor=False):
+    def __init__(self, obj, stringsasfactor=False, checknames=False):
         """ Create a new data frame.
 
         :param obj: object inheriting from rpy2.rinterface.SexpVector,
-                    or inheriting from TaggedList
-                    or a mapping name -> value
+            or inheriting from TaggedList or a mapping name -> value
         :param stringsasfactors: Boolean indicating whether vectors
-                    of strings should be turned to vectors. Note
-                    that factors will not be turned to string vectors.
+            of strings should be turned to vectors. Note that factors
+            will not be turned to string vectors.
+        :param checknames: Boolean indicating whether column names
+            should be transformed to R syntactically valid names.
         """
         if isinstance(obj, rinterface.ListSexpVector):
             if obj.typeof != rinterface.RTYPES.VECSXP:
@@ -1209,7 +1250,8 @@ class DataFrame(ListVector):
                           'conflicting with named parameter '
                           'in underlying R function "data.frame()".')
         else:
-            kv.append(('stringsAsFactors', stringsasfactor))
+            kv.extend((('stringsAsFactors', stringsasfactor),
+                       ('check.names', checknames)))
 
         # Call R's data frame constructor
         kv = tuple(kv)
@@ -1324,8 +1366,8 @@ class DataFrame(ListVector):
         res = utils_ri['head'](self, *args, **kwargs)
         return conversion.rpy2py(res)
 
-    @staticmethod
-    def from_csvfile(path, header=True, sep=',',
+    @classmethod
+    def from_csvfile(cls, path, header=True, sep=',',
                      quote='"', dec='.',
                      row_names=rinterface.MissingArg,
                      col_names=rinterface.MissingArg,
@@ -1369,8 +1411,7 @@ class DataFrame(ListVector):
                                      'comment.char': comment_char,
                                      'na.strings': na_strings,
                                      'as.is': as_is})
-        res = conversion.rpy2py(res)
-        return res
+        return cls(res)
 
     def to_csvfile(self, path, quote=True, sep=',',
                    eol=os.linesep, na='NA', dec='.',
